from distutils.core import setup

setup(
    name = 'bucket_save_upload',
    packages = ['bucket_save_upload'],
    version = 'v1.0.2',
    description = "Save and upload to GCS buckets in a parallel fashion.",
    author="Rennan Valadares",
    author_email="rennanvoa2@gmail.com",
    url = 'https://github.com/NeobrainData/bucket-save',
    download_url = 'https://github.com/NeobrainData/bucket-save-upload/archive/refs/tags/v1.0.2.tar.gz',
    keywords = ['GCS', 'Google Cloud Storage', 'bucket', 'upload'],
    classifiers = [],
)